require "test/player"
require "test/collide"
require "test/trigger"
require "test/prop"
require "test/messages"
require "test/prop_spawner"
require "test/multisource"
require "test/bullet"


object_manager = {}
object_manager.object_list = {}
object_manager.remove_list = {}
object_manager.collision_list = {}
object_manager.new_objects = {}

love.physics.setMeter(64)


text       = ""   -- we'll use this to put info text on the screen later
persisting = 0    -- we'll use this to store the state of repeated callback calls
	
object1 = {}
local x,y = 0,0

function object_manager:init()

	object_manager.world = love.physics.newWorld(0, 9.81*64,true)
	object_manager.world:setCallbacks(beginContact,endContact,preSolve,postSolve)

end

function object_manager:create_object(name,object,x,y,w,h,args)
	table.insert(object_manager.new_objects,{name = name, object = object, x = x , y = y , w = w, h = h , args = args})
end


function object_manager:create_objects()
	local delete_obj = {}
	for i, obj in pairs(object_manager.new_objects) do
	
		if(obj.object == "player") then
			object_manager:add_object(player:new(obj.x,obj.y))
		elseif(obj.object == "bullet") then
			object_manager:add_object(bullet:new(obj.x,obj.y))
		elseif(obj.object == "prop") then
			object_manager:add_object(prop:new(obj.name,obj.x,obj.y,obj.args))
		elseif(obj.object == "collide") then
			object_manager:add_boundBox(collide:new(obj.x,obj.y,obj.w,obj.h,obj.args))
		elseif(obj.object == "trigger") then
			object_manager:add_trigger(trigger:new(obj.name,obj.x,obj.y,obj.w,obj.h,obj.args))
		elseif(obj.object == "prop_spawner") then
			object_manager:add_object(prop_spawner:new(obj.name,obj.x,obj.y,obj.args))
		elseif(obj.object == "multisource") then
			object_manager:add_object(multisource:new(obj.name,obj.x,obj.y,obj.args))
		end
		table.insert(delete_obj,i)
	end
	for v, o in pairs(delete_obj) do
		object_manager.new_objects[o] = nil
	end
	
end

function object_manager:add_boundBox(bd)
		table.insert(object_manager.collision_list,bd)
end

function object_manager:add_object(ob)
	print(ob.name)
	table.insert(object_manager.object_list,ob)

end
function object_manager:add_trigger(ob)
	table.insert(object_manager.object_list,ob)
end

function object_manager:get_objects()
	return object_manager.object_list;
end

function object_manager:get_tofollow()
	return x,y
end

function object_manager:Update(dt)
	
	object_manager.world:update(dt)
	object_manager:create_objects()
	
	for i,v in ipairs(object_manager:get_objects()) do
		if(v.alive == true) then
			if(v.type == "player") then
				x =  v.x + v.origin_x - love.graphics.getWidth() / 2
				y =  v.y + v.origin_y - love.graphics.getHeight() / 2
			end
			if(v.update == true )then
				v:Update(dt)
			end
			else
			if(v.body ~= nil) then
				v.body:destroy()
			end
			table.insert(self.remove_list,i)
		end
	end
	
	for i,v in ipairs(self.remove_list) do
		self.object_list[v] = nil
		table.remove(self.object_list,v)
		
		table.remove(self.remove_list,i)
	end
	
	
end

function object_manager:getObjectByName(name)

	for i,v in ipairs(object_manager.object_list) do
		if(v.name == name) then
			return i
		end
	end
end

function object_manager:getTypeByName(name)

	if(name == "World") then
		return "World"
	end
	for i,v in ipairs(object_manager.object_list) do
		

		if(v.name == name) then
			return v.type
		end
	end
	
end


function beginContact(a, b, coll)

if(object_manager.object_list[object_manager:getObjectByName(a:getUserData().name)] ~= nil) then
	if(object_manager.object_list[object_manager:getObjectByName(a:getUserData().name)].c_inf == true) then
		object_manager.object_list[object_manager:getObjectByName(a:getUserData().name)]:collide(a,b,c)
	end
end

if(object_manager.object_list[object_manager:getObjectByName(b:getUserData().name)] ~= nil) then
	if(object_manager.object_list[object_manager:getObjectByName(b:getUserData().name)].c_inf == true) then
		object_manager.object_list[object_manager:getObjectByName(b:getUserData().name)]:collide(b,a,c)
	end
end
--if(a_name ~= "World" and object_manager.object_list[object_manager:getObjectByName(a_name)].c_inf == true) then

	--print(a_name .. " is not world,wants to be informed, and its informed of the collision")
--	msg_manager:send_message(b_name,a_name,{reason="c_start"})
--end



--if(b_name ~= "World" and object_manager.object_list[object_manager:getObjectByName(b_name)].c_inf == true) then

--	print(b_name .. " is not world,wants to be informed, and its informed of the collision")
	
--	msg_manager:send_message(a_name,b_name,{reason="c_start"})
--end


end
function endContact(a, b, coll)

end

function preSolve(a, b, coll)

end

function postSolve(a, b, coll)
    
end
